@@ -15,8 +15,7 @@
 
 """Glance Store exception subclasses"""
 
-import six
-import six.moves.urllib.parse as urlparse
+import urllib.parse
 
 from glance_store.i18n import _
 
@@ -31,7 +30,7 @@ class UnsupportedBackend(BackendException):
 
 class RedirectException(Exception):
     def __init__(self, url):
-        self.url = urlparse.urlparse(url)
+        self.url = urllib.parse.urlparse(url)
 
 
 class GlanceStoreException(Exception):
@@ -51,16 +50,10 @@ class GlanceStoreException(Exception):
             if kwargs:
                 message = message % kwargs
         except Exception:
-                pass
+            pass
         self.msg = message
         super(GlanceStoreException, self).__init__(message)
 
-    def __unicode__(self):
-        # NOTE(flwang): By default, self.msg is an instance of Message, which
-        # can't be converted by str(). Based on the definition of
-        # __unicode__, it should return unicode always.
-        return six.text_type(self.msg)
-
 
 class MissingCredentialError(GlanceStoreException):
     message = _("Missing required credential: %(required)s")
@@ -181,3 +174,8 @@ class HasSnapshot(GlanceStoreException):
 class InUseByStore(GlanceStoreException):
     message = _("The image cannot be deleted because it is in use through "
                 "the backend store outside of Glance.")
+
+
+class HostNotInitialized(GlanceStoreException):
+    message = _("The glance cinder store host %(host)s which will used to "
+                "perform nfs mount/umount operations isn't initialized.")
