@@ -16,7 +16,6 @@
 import contextlib
 import hashlib
 import io
-import math
 import os
 from unittest import mock
 
@@ -32,7 +31,6 @@ from oslo_concurrency import processutils
 from oslo_utils.secretutils import md5
 from oslo_utils import units
 
-from glance_store._drivers.cinder import scaleio
 from glance_store.common import attachment_state_manager
 from glance_store.common import cinder_utils
 from glance_store import exceptions
@@ -579,6 +577,55 @@ class TestCinderStoreBase(object):
             if is_multi_store:
                 self.assertEqual(backend, metadata["store"])
 
+    def _test_cinder_add_size_validation(
+            self, fake_volume, volume_file, verifier=None, size_kb=5,
+            backend='glance_store', is_multi_store=False, oversized=False):
+        expected_image_id = str(uuid.uuid4())
+        expected_size = size_kb * units.Ki
+        if oversized:
+            # 1 KB over
+            self.store.WRITE_CHUNKSIZE = 1024
+            expected_file_contents = b"*" * (expected_size + 1024)
+            image_file = io.BytesIO(expected_file_contents)
+            expected_error = "Size exceeds: expected"
+        else:
+            # 1 KB less
+            expected_file_contents = b"*" * (expected_size - 1024)
+            image_file = io.BytesIO(expected_file_contents)
+            expected_error = "Size mismatch: expected"
+
+        if is_multi_store:
+            # Default backend is 'glance_store' for single store but in case
+            # of multi store, if the backend option is not passed, we should
+            # assign it to the default i.e. 'cinder1'
+            if backend == 'glance_store':
+                backend = 'cinder1'
+        self.config(cinder_volume_type='some_type', group=backend)
+
+        fake_client = mock.MagicMock(auth_token=None, management_url=None)
+        fake_volume.manager.get.return_value = fake_volume
+        fake_volumes = mock.MagicMock(create=mock.Mock(
+            return_value=fake_volume))
+
+        @contextlib.contextmanager
+        def fake_open(client, volume, mode):
+            self.assertEqual('wb', mode)
+            yield volume_file
+
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
+                mock.patch.object(self.store, '_open_cinder_volume',
+                                  side_effect=fake_open):
+            mock_cc.return_value = mock.MagicMock(client=fake_client,
+                                                  volumes=fake_volumes)
+            self.assertRaisesRegex(
+                exceptions.Invalid, expected_error,
+                self.store.add, expected_image_id, image_file,
+                expected_size, self.hash_algo, self.context, verifier)
+
+            fake_volume.delete.assert_called_once()
+            self.assertEqual(image_file.tell(),
+                             len(expected_file_contents))
+
     def test_cinder_add_volume_not_found(self):
         image_file = mock.MagicMock()
         fake_image_id = str(uuid.uuid4())
@@ -797,38 +844,6 @@ class TestCinderStoreBase(object):
             self.assertRaises(exceptions.BackendException, self.store.delete,
                               loc, context=self.context)
 
-    def test__get_device_size(self):
-        fake_data = b"fake binary data"
-        fake_len = int(math.ceil(float(len(fake_data)) / units.Gi))
-        fake_file = io.BytesIO(fake_data)
-        dev_size = scaleio.ScaleIOBrickConnector._get_device_size(fake_file)
-        self.assertEqual(fake_len, dev_size)
-
-    @mock.patch.object(time, 'sleep')
-    def test__wait_resize_device_resized(self, mock_sleep):
-        fake_vol = mock.MagicMock()
-        fake_vol.size = 2
-        fake_file = io.BytesIO(b"fake binary data")
-        with mock.patch.object(
-                scaleio.ScaleIOBrickConnector,
-                '_get_device_size') as mock_get_dev_size:
-            mock_get_dev_size.side_effect = [1, 2]
-            scaleio.ScaleIOBrickConnector._wait_resize_device(
-                fake_vol, fake_file)
-
-    @mock.patch.object(time, 'sleep')
-    def test__wait_resize_device_fails(self, mock_sleep):
-        fake_vol = mock.MagicMock()
-        fake_vol.size = 2
-        fake_file = io.BytesIO(b"fake binary data")
-        with mock.patch.object(
-                scaleio.ScaleIOBrickConnector, '_get_device_size',
-                return_value=1):
-            self.assertRaises(
-                exceptions.BackendException,
-                scaleio.ScaleIOBrickConnector._wait_resize_device,
-                fake_vol, fake_file)
-
     def test_process_specs(self):
         self.location.process_specs()
         self.assertEqual('cinder', self.location.scheme)
