@@ -16,6 +16,7 @@
 """Tests the Multiple S3 backend store"""
 
 import hashlib
+import io
 from unittest import mock
 import uuid
 
@@ -24,9 +25,7 @@ import botocore
 from botocore import exceptions as boto_exceptions
 from botocore import stub
 from oslo_config import cfg
-from oslo_utils.secretutils import md5
 from oslo_utils import units
-import six
 
 import glance_store as store
 from glance_store._drivers import s3
@@ -90,6 +89,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
                     s3_store_access_key='user',
                     s3_store_secret_key='key',
                     s3_store_host='https://s3-region1.com',
+                    s3_store_region_name='custom_region_name',
+                    s3_store_cacert='path/to/cert/bundle.pem',
                     s3_store_bucket='glance',
                     s3_store_large_object_size=S3_CONF[
                         's3_store_large_object_size'
@@ -132,12 +133,46 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             self.assertRaises(boto_exceptions.InvalidDNSNameError,
                               self.store.get, loc)
 
+    @mock.patch('glance_store.location.Location')
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_client_custom_region_name(self, mock_client, mock_loc):
+        """Test a custom s3_store_region_name in config"""
+        mock_loc.accesskey = 'abcd'
+        mock_loc.secretkey = 'efgh'
+        mock_loc.bucket = 'bucket1'
+        self.store._create_s3_client(mock_loc)
+        mock_client.assert_called_with(
+            config=mock.ANY,
+            endpoint_url='https://s3-region1.com',
+            region_name='custom_region_name',
+            service_name='s3',
+            use_ssl=False,
+            verify='path/to/cert/bundle.pem',
+        )
+
+    @mock.patch('glance_store.location.Location')
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_client_custom_ca_cert_bundle(self, mock_client, mock_loc):
+        """Test a custom s3_store_cacert in config"""
+        mock_loc.accesskey = 'abcd'
+        mock_loc.secretkey = 'efgh'
+        mock_loc.bucket = 'bucket1'
+        self.store._create_s3_client(mock_loc)
+        mock_client.assert_called_with(
+            config=mock.ANY,
+            endpoint_url='https://s3-region1.com',
+            region_name='custom_region_name',
+            service_name='s3',
+            use_ssl=False,
+            verify='path/to/cert/bundle.pem',
+        )
+
     @mock.patch.object(boto3.session.Session, "client")
     def test_get(self, mock_client):
         """Test a "normal" retrieval of an image in chunks."""
         bucket, key = 'glance', FAKE_UUID
         fixture_object = {
-            'Body': six.BytesIO(b"*" * FIVE_KB),
+            'Body': io.BytesIO(b"*" * FIVE_KB),
             'ContentLength': FIVE_KB
         }
         fake_s3_client = botocore.session.get_session().create_client('s3')
@@ -211,8 +246,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         # 5KiB is smaller than WRITE_CHUNKSIZE
         expected_s3_size = FIVE_KB
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = md5(expected_s3_contents,
-                                usedforsecurity=False).hexdigest()
+        expected_checksum = hashlib.md5(expected_s3_contents,
+                                        usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
@@ -220,7 +255,7 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -256,6 +291,119 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             self.assertEqual(expected_checksum, checksum)
             self.assertEqual(expected_multihash, multihash)
 
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_add_singlepart_size_exceeding_max_size(self, mock_client):
+        """Test size validation during add for singlepart upload."""
+        self.store.WRITE_CHUNKSIZE = 1024
+        expected_image_id = str(uuid.uuid4())
+        # Provide a smaller expected size than actual data to trigger
+        # size mismatch
+        expected_s3_size = FIVE_KB
+        # 5KB more than expected
+        actual_s3_contents = b"*" * (expected_s3_size * 2)
+        image_s3 = io.BytesIO(actual_s3_contents)
+
+        fake_s3_client = botocore.session.get_session().create_client('s3')
+
+        with stub.Stubber(fake_s3_client) as stubber:
+            stubber.add_response(method='head_bucket',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket']
+                                 })
+            stubber.add_client_error(method='head_object',
+                                     service_error_code='404',
+                                     service_message='',
+                                     expected_params={
+                                         'Bucket': S3_CONF['s3_store_bucket'],
+                                         'Key': expected_image_id
+                                     })
+            stubber.add_response(method='put_object',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket'],
+                                     'Key': expected_image_id,
+                                     'Body': botocore.stub.ANY
+                                 })
+
+            mock_client.return_value = fake_s3_client
+
+            # Expect an exception due to size mismatch
+            self.assertRaisesRegex(exceptions.Invalid,
+                                   'Size exceeds: expected',
+                                   self.store.add,
+                                   expected_image_id, image_s3,
+                                   expected_s3_size, self.hash_algo)
+
+            # Verify that the stream's position reflects the number of bytes
+            # read, which should be exactly at expected_file_size plus the
+            # last buffer size read.
+            actual_read_size = image_s3.tell()
+            expected_read = expected_s3_size + self.store.WRITE_CHUNKSIZE
+            self.assertEqual(actual_read_size, expected_read,
+                             "The stream was not read only up to the expected "
+                             "size.")
+
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_add_singlepart_write_less_than_declared(self, mock_client):
+        """Test size validation during add for singlepart upload."""
+        expected_image_id = str(uuid.uuid4())
+        # Provide a smaller expected size than actual data to trigger
+        # size mismatch
+        expected_s3_size = FIVE_KB
+        # 100 bytes smaller than expected
+        actual_s3_contents = b"*" * (FIVE_KB - 100)
+        image_s3 = io.BytesIO(actual_s3_contents)
+
+        fake_s3_client = botocore.session.get_session().create_client('s3')
+
+        with stub.Stubber(fake_s3_client) as stubber:
+            stubber.add_response(method='head_bucket',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket']
+                                 })
+            stubber.add_client_error(method='head_object',
+                                     service_error_code='404',
+                                     service_message='',
+                                     expected_params={
+                                         'Bucket': S3_CONF['s3_store_bucket'],
+                                         'Key': expected_image_id
+                                     })
+            stubber.add_response(method='head_bucket',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket']
+                                 })
+            # Add a second head_object call to match the code's behavior
+            stubber.add_client_error(method='head_object',
+                                     service_error_code='404',
+                                     service_message='',
+                                     expected_params={
+                                         'Bucket': S3_CONF['s3_store_bucket'],
+                                         'Key': expected_image_id
+                                     })
+            stubber.add_response(method='put_object',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket'],
+                                     'Key': expected_image_id,
+                                     'Body': botocore.stub.ANY
+                                 })
+
+            mock_client.return_value = fake_s3_client
+
+            # Expect an exception due to size mismatch
+            self.assertRaisesRegex(exceptions.Invalid,
+                                   'Size mismatch: expected',
+                                   self.store.add,
+                                   expected_image_id, image_s3,
+                                   expected_s3_size, self.hash_algo)
+            # The input buffer should be fully read
+            # depending on implementation
+            total_input_size = len(actual_s3_contents)
+            self.assertEqual(image_s3.tell(), total_input_size)
+
     @mock.patch.object(boto3.session.Session, "client")
     def test_add_singlepart_bigger_than_write_chunk(self, mock_client):
         """Test that we can add an image via the s3 backend."""
@@ -264,8 +412,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         # but smaller than s3_store_large_object_size
         expected_s3_size = 8 * units.Mi
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = md5(expected_s3_contents,
-                                usedforsecurity=False).hexdigest()
+        expected_checksum = hashlib.md5(expected_s3_contents,
+                                        usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
@@ -273,7 +421,7 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -318,8 +466,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = FIVE_KB
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = md5(expected_s3_contents,
-                                usedforsecurity=False).hexdigest()
+        expected_checksum = hashlib.md5(expected_s3_contents,
+                                        usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
@@ -327,7 +475,7 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             'http://s3-region2.com',
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -369,7 +517,7 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = FIVE_KB
         expected_s3_contents = b"*" * expected_s3_size
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
         verifier = mock.MagicMock(name='mock_verifier')
@@ -392,8 +540,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = 16 * units.Mi
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = md5(expected_s3_contents,
-                                usedforsecurity=False).hexdigest()
+        expected_checksum = hashlib.md5(expected_s3_contents,
+                                        usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
@@ -401,7 +549,7 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -478,12 +626,143 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
             self.assertEqual(expected_checksum, checksum)
             self.assertEqual(expected_multihash, multihash)
 
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_add_multipart_size_exceeding_max_size(self, mock_client):
+        """Test size validation during multipart upload."""
+        expected_image_id = str(uuid.uuid4())
+        expected_s3_size = 15 * units.Mi
+        self.store.WRITE_CHUNKSIZE = 5 * units.Mi
+        # Actual data is larger than expected to trigger size validation
+        # failure, 5MB larger
+        total_size = expected_s3_size + self.store.WRITE_CHUNKSIZE
+        expected_s3_contents = b"*" * total_size
+        image_s3 = io.BytesIO(expected_s3_contents)
+
+        fake_s3_client = botocore.session.get_session().create_client('s3')
+        # Patch abort_multipart_upload on the client to a Mock
+        fake_s3_client.abort_multipart_upload = mock.Mock()
+        with stub.Stubber(fake_s3_client) as stubber:
+            stubber.add_response(method='head_bucket',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket']
+                                 })
+            stubber.add_client_error(method='head_object',
+                                     service_error_code='404',
+                                     service_message='',
+                                     expected_params={
+                                         'Bucket': S3_CONF['s3_store_bucket'],
+                                         'Key': expected_image_id
+                                     })
+            stubber.add_response(method='create_multipart_upload',
+                                 service_response={
+                                     "Bucket": S3_CONF['s3_store_bucket'],
+                                     "Key": expected_image_id,
+                                     "UploadId": 'UploadId'
+                                 },
+                                 expected_params={
+                                     "Bucket": S3_CONF['s3_store_bucket'],
+                                     "Key": expected_image_id,
+                                 })
+            stubber.add_response(method='abort_multipart_upload',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket'],
+                                     'Key': expected_image_id,
+                                     'UploadId': 'UploadId'
+                                 })
+
+            mock_client.return_value = fake_s3_client
+            # Expect an exception due to size mismatch
+            self.assertRaisesRegex(
+                exceptions.Invalid, "Size exceeds: expected",
+                self.store.add, expected_image_id, image_s3,
+                expected_s3_size, self.hash_algo)
+
+            # Verify that the stream's position reflects the number of bytes
+            # read, which should be exactly at expected_file_size plus the
+            # last buffer size read.
+            actual_read_size = image_s3.tell()
+            self.assertEqual(actual_read_size, total_size,
+                             "The stream was not read only up to the expected "
+                             "size.")
+
+            # Assert that abort_multipart_upload was called
+            fake_s3_client.abort_multipart_upload.assert_called_once_with(
+                Bucket=S3_CONF['s3_store_bucket'],
+                Key=expected_image_id,
+                UploadId='UploadId'
+            )
+
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_add_multipart_write_less_than_declared(self, mock_client):
+        """Test size validation during multipart upload."""
+        expected_image_id = str(uuid.uuid4())
+        expected_s3_size = 16 * units.Mi
+        # Actual data is less than expected to trigger size validation
+        # failure, 1MB less
+        expected_s3_contents = b"*" * (expected_s3_size - 1 * units.Mi)
+        image_s3 = io.BytesIO(expected_s3_contents)
+
+        fake_s3_client = botocore.session.get_session().create_client('s3')
+        # Patch abort_multipart_upload on the client to a Mock
+        fake_s3_client.abort_multipart_upload = mock.Mock()
+        with stub.Stubber(fake_s3_client) as stubber:
+            stubber.add_response(method='head_bucket',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket']
+                                 })
+            stubber.add_client_error(method='head_object',
+                                     service_error_code='404',
+                                     service_message='',
+                                     expected_params={
+                                         'Bucket': S3_CONF['s3_store_bucket'],
+                                         'Key': expected_image_id
+                                     })
+            stubber.add_response(method='create_multipart_upload',
+                                 service_response={
+                                     "Bucket": S3_CONF['s3_store_bucket'],
+                                     "Key": expected_image_id,
+                                     "UploadId": 'UploadId'
+                                 },
+                                 expected_params={
+                                     "Bucket": S3_CONF['s3_store_bucket'],
+                                     "Key": expected_image_id,
+                                 })
+            stubber.add_response(method='abort_multipart_upload',
+                                 service_response={},
+                                 expected_params={
+                                     'Bucket': S3_CONF['s3_store_bucket'],
+                                     'Key': expected_image_id,
+                                     'UploadId': 'UploadId'
+                                 })
+
+            mock_client.return_value = fake_s3_client
+            # Expect an exception due to size mismatch
+            self.assertRaisesRegex(exceptions.Invalid,
+                                   'Size mismatch: expected',
+                                   self.store.add,
+                                   expected_image_id, image_s3,
+                                   expected_s3_size, self.hash_algo)
+
+            # The input buffer should be fully read depending on implementation
+            total_input_size = len(expected_s3_contents)
+            self.assertEqual(image_s3.tell(), total_input_size)
+
+            # Assert that abort_multipart_upload was called
+            fake_s3_client.abort_multipart_upload.assert_called_once_with(
+                Bucket=S3_CONF['s3_store_bucket'],
+                Key=expected_image_id,
+                UploadId='UploadId'
+            )
+
     @mock.patch.object(boto3.session.Session, "client")
     def test_add_already_existing(self, mock_client):
         """Tests that adding an image with an existing identifier raises an
         appropriate exception
         """
-        image_s3 = six.BytesIO(b"never_gonna_make_it")
+        image_s3 = io.BytesIO(b"never_gonna_make_it")
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
