@@ -16,16 +16,15 @@
 """Tests the VMware Datastore backend store"""
 
 import hashlib
+import io
 from unittest import mock
 import uuid
 
-from oslo_utils import secretutils
 from oslo_utils import units
 from oslo_vmware import api
 from oslo_vmware import exceptions as vmware_exceptions
 from oslo_vmware.objects import datacenter as oslo_datacenter
 from oslo_vmware.objects import datastore as oslo_datastore
-import six
 
 import glance_store._drivers.vmware_datastore as vm_store
 from glance_store import backend
@@ -105,7 +104,7 @@ class TestStore(base.StoreBaseTest,
         self.hash_algo = 'sha256'
 
     def _mock_http_connection(self):
-        return mock.patch('six.moves.http_client.HTTPConnection')
+        return mock.patch('http.client.HTTPConnection')
 
     @mock.patch('oslo_vmware.api.VMwareAPISession')
     def test_get(self, mock_api_session):
@@ -146,14 +145,14 @@ class TestStore(base.StoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_size = FIVE_KB
         expected_contents = b"*" * expected_size
-        hash_code = secretutils.md5(expected_contents, usedforsecurity=False)
+        hash_code = hashlib.md5(expected_contents, usedforsecurity=False)
         expected_checksum = hash_code.hexdigest()
         sha256_code = hashlib.sha256(expected_contents)
         expected_multihash = sha256_code.hexdigest()
         fake_size.__get__ = mock.Mock(return_value=expected_size)
         expected_cookie = 'vmware_soap_session=fake-uuid'
         fake_cookie.return_value = expected_cookie
-        expected_headers = {'Content-Length': six.text_type(expected_size),
+        expected_headers = {'Content-Length': str(expected_size),
                             'Cookie': expected_cookie}
         with mock.patch('hashlib.md5') as md5:
             with mock.patch('hashlib.new') as fake_new:
@@ -164,7 +163,7 @@ class TestStore(base.StoreBaseTest,
                     VMWARE_DS['vmware_store_image_dir'],
                     expected_image_id,
                     VMWARE_DS['vmware_datastores'])
-                image = six.BytesIO(expected_contents)
+                image = io.BytesIO(expected_contents)
                 with mock.patch('requests.Session.request') as HttpConn:
                     HttpConn.return_value = utils.fake_response()
                     location, size, checksum, multihash, _ = self.store.add(
@@ -191,7 +190,7 @@ class TestStore(base.StoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_size = FIVE_KB
         expected_contents = b"*" * expected_size
-        hash_code = secretutils.md5(expected_contents, usedforsecurity=False)
+        hash_code = hashlib.md5(expected_contents, usedforsecurity=False)
         expected_checksum = hash_code.hexdigest()
         sha256_code = hashlib.sha256(expected_contents)
         expected_multihash = sha256_code.hexdigest()
@@ -205,7 +204,7 @@ class TestStore(base.StoreBaseTest,
                     VMWARE_DS['vmware_store_image_dir'],
                     expected_image_id,
                     VMWARE_DS['vmware_datastores'])
-                image = six.BytesIO(expected_contents)
+                image = io.BytesIO(expected_contents)
                 with mock.patch('requests.Session.request') as HttpConn:
                     HttpConn.return_value = utils.fake_response()
                     location, size, checksum, multihash, _ = self.store.add(
@@ -224,7 +223,7 @@ class TestStore(base.StoreBaseTest,
         image_id = str(uuid.uuid4())
         size = FIVE_KB
         contents = b"*" * size
-        image = six.BytesIO(contents)
+        image = io.BytesIO(contents)
         with mock.patch('requests.Session.request') as HttpConn:
             HttpConn.return_value = utils.fake_response()
             self.store.add(image_id, image, size, self.hash_algo,
@@ -240,7 +239,7 @@ class TestStore(base.StoreBaseTest,
         image_id = str(uuid.uuid4())
         size = FIVE_KB
         contents = b"*" * size
-        image = six.BytesIO(contents)
+        image = io.BytesIO(contents)
         with mock.patch('requests.Session.request') as HttpConn:
             HttpConn.return_value = utils.fake_response()
             self.store.add(image_id, image, 0, self.hash_algo,
@@ -303,9 +302,9 @@ class TestStore(base.StoreBaseTest,
 
     def test_reader_full(self):
         content = b'XXX'
-        image = six.BytesIO(content)
-        expected_checksum = secretutils.md5(content,
-                                            usedforsecurity=False).hexdigest()
+        image = io.BytesIO(content)
+        expected_checksum = hashlib.md5(content,
+                                        usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(content).hexdigest()
         reader = vm_store._Reader(image, self.hash_algo)
         ret = reader.read()
@@ -316,9 +315,9 @@ class TestStore(base.StoreBaseTest,
 
     def test_reader_partial(self):
         content = b'XXX'
-        image = six.BytesIO(content)
-        expected_checksum = secretutils.md5(b'X',
-                                            usedforsecurity=False).hexdigest()
+        image = io.BytesIO(content)
+        expected_checksum = hashlib.md5(b'X',
+                                        usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(b'X').hexdigest()
         reader = vm_store._Reader(image, self.hash_algo)
         ret = reader.read(1)
@@ -329,7 +328,7 @@ class TestStore(base.StoreBaseTest,
 
     def test_reader_with_verifier(self):
         content = b'XXX'
-        image = six.BytesIO(content)
+        image = io.BytesIO(content)
         verifier = mock.MagicMock(name='mock_verifier')
         reader = vm_store._Reader(image, self.hash_algo, verifier)
         reader.read()
@@ -414,7 +413,7 @@ class TestStore(base.StoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_size = FIVE_KB
         expected_contents = b"*" * expected_size
-        image = six.BytesIO(expected_contents)
+        image = io.BytesIO(expected_contents)
         self.session = mock.Mock()
         with mock.patch('requests.Session.request') as HttpConn:
             HttpConn.return_value = utils.fake_response(status_code=401)
@@ -430,7 +429,7 @@ class TestStore(base.StoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_size = FIVE_KB
         expected_contents = b"*" * expected_size
-        image = six.BytesIO(expected_contents)
+        image = io.BytesIO(expected_contents)
         self.session = mock.Mock()
         with self._mock_http_connection() as HttpConn:
             HttpConn.return_value = utils.fake_response(status_code=500,
@@ -473,7 +472,7 @@ class TestStore(base.StoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_size = FIVE_KB
         expected_contents = b"*" * expected_size
-        image = six.BytesIO(expected_contents)
+        image = io.BytesIO(expected_contents)
         self.session = mock.Mock()
         with mock.patch('requests.Session.request') as HttpConn:
             HttpConn.request.side_effect = IOError
